// post.service.ts
import _ from 'lodash';
import { Repository } from 'typeorm';

import {
  BadRequestException,
  Injectable,
  NotFoundException,
  UnauthorizedException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Post } from './entities/post.entity';
/**
 * 리포지토리
 * 리포지토리로 도메인 로직이 데이터베이스의 세부 구현에서 분리되므로 코드의 유지 보수성과 확장성이 향상
 * 엔티티와 데이터베이스 간의 중간 계층
 *  프로그래머가 데이터베이스와의 통신 과정을 알지 못하더라도 
 *  추상화 된 리포지토리의 함수를 사용하여 
 *  데이터베이스에서 원하는 결과를 얻을 수 있도록 도움
 */

import { CreatePostDto } from './dto/create-post.dto';
import { UpdatePostDto } from './dto/update-post.dto';
import { RemovePostDto } from './dto/remove-post.dto';

@Injectable()
export class PostService {
    // 생성자에서 @InjectRepository 어노테이션을 사용하여 리포지토리를 주입
    constructor(
      @InjectRepository(Post) private postRepository: Repository<Post>,
    ) {}
  
    async create(createPostDto: CreatePostDto) {
      return (await this.postRepository.save(createPostDto)).id;
  }
    async findAll() {
      return await this.postRepository.find({
         // 삭제일시컬럼이 비어있는 것만 가져오기 = 삭제되지 않은 Post만 가져오기
        where: { deletedAt: null },
        select: ['id', 'title', 'updatedAt'],
      });
    }
  
    async findOne(id: number) {
      /**- `_.isNaN` 함수
       * id가 number 타입으로 변환할 수 없는 경우
       * 컨트롤러에서 전달되는 +id 값이 NaN이 되기 때문에 사용.
       * 
       * ID값을 잘못 준 경우에 잘못된 요청을 하였다고 알려주기
       *   `BadRequestException` 예외를 던진다 
       * */
      if (_.isNaN(id)) {
        throw new BadRequestException('게시물 ID가 잘못되었습니다.');
      }
  
      return await this.postRepository.findOne({
        where: { id, deletedAt: null }, // 삭제되지 않은 해당id의 post 가져오기
        select: ['title', 'content', 'updatedAt'],
      });
  }


  async update(id: number, updatePostDto: UpdatePostDto) {
    if (_.isNaN(id)) {
      throw new BadRequestException('게시물 ID가 잘못되었습니다.');
    }

    const { content, password } = updatePostDto;
    const post = await this.postRepository.findOne({
      select: ['password'],
      where: { id },
    });

    if (_.isNil(post)) {
      throw new NotFoundException('게시물을 찾을 수 없습니다.');
    }

   // post.password : select가 false로 지정이 되어있으므로 명시적으로 select를 하지 않으면 password 값을 받아올 수 없기 때문 
    if (!_.isNil(post.password) && post.password !== password) {
    // UnauthorizedException 예외 : 401   비밀번호가 틀렸다면 수정할 권한이 없다는 예외를 던지는 것
      throw new UnauthorizedException('비밀번호가 일치하지 않습니다.'); // 401
    }

    await this.postRepository.update({ id }, { content });
  }


  async remove(id: number, removePostDto: RemovePostDto) {
    if (_.isNaN(id)) {
      throw new BadRequestException('게시물 ID가 잘못되었습니다.');
    }

    const { password } = removePostDto;

    const post = await this.postRepository.findOne({
      select: ['password'],
      where: { id },
    });

    if (_.isNil(post)) {
      throw new NotFoundException('게시물을 찾을 수 없습니다.');
    }

    // post.password : select가 false로 지정이 되어있으므로 명시적으로 select를 하지 않으면 password 값을 받아올 수 없기 때문 
    if (!_.isNil(post.password) && post.password !== password) {
    // UnauthorizedException 예외 : 401   비밀번호가 틀렸다면 수정할 권한이 없다는 예외를 던지는 것
      throw new UnauthorizedException('비밀번호가 일치하지 않습니다.'); // 401
    }

    // softDelete는 DB상에서 지워지는 것이 아니라 deletedAt에 지워진 날짜만 마킹
    return this.postRepository.softDelete({ id });
  }
}